From 6132d0aa9387e9a23ac7608d269eed7c4f5424e9 Mon Sep 17 00:00:00 2001
From: Paul Bakker
Date: Wed, 4 Jul 2012 17:10:40 +0000
Subject: [PATCH] - Added Blowfish to generic cipher layer - Renamed
POLARSSL_MODE_CFB128 to POLARSSL_MODE_CFB
---
ChangeLog | 4 +-
include/polarssl/blowfish.h | 2 +-
include/polarssl/cipher.h | 12 +-
include/polarssl/cipher_wrap.h | 12 +
library/cipher.c | 54 +++-
library/cipher_wrap.c | 128 +++++++-
tests/CMakeLists.txt | 1 +
tests/Makefile | 13 +-
tests/suites/test_suite_cipher.blowfish.data | 318 +++++++++++++++++++
9 files changed, 525 insertions(+), 19 deletions(-)
create mode 100644 tests/suites/test_suite_cipher.blowfish.data
diff --git a/ChangeLog b/ChangeLog
index ec0bc0986..f86254458 100644
--- a/ChangeLog
+++ b/ChangeLog
@@ -22,7 +22,7 @@ Features
* Added X509 CA Path support
* Added Thumb assembly optimizations
* Added DEFLATE compression support as per RFC3749 (requires zlib)
- * Added base blowfish algorithm
+ * Added blowfish algorithm (Generic and cipher layer)
Changes
* Removed redundant POLARSSL_DEBUG_MSG define
@@ -32,6 +32,8 @@ Changes
* Moved out_msg to out_hdr + 32 to support hardware acceleration
* Changed certificate verify behaviour to comply with RFC 6125 section 6.3
to not match CN if subjectAltName extension is present (Closes ticket #56)
+ * Cipher layer cipher_mode_t POLARSSL_MODE_CFB128 is renamed to
+ POLARSSL_MODE_CFB, to also handle different block size CFB modes.
Bugfix
* Fixed handling error in mpi_cmp_mpi() on longer B values (found by
diff --git a/include/polarssl/blowfish.h b/include/polarssl/blowfish.h
index 89f06bd5a..dc469d0b8 100644
--- a/include/polarssl/blowfish.h
+++ b/include/polarssl/blowfish.h
@@ -54,7 +54,7 @@ extern "C" {
#endif
/**
- * \brief Blowfish key schedule (encryption)
+ * \brief Blowfish key schedule
*
* \param ctx Blowfish context to be initialized
* \param key encryption key
diff --git a/include/polarssl/cipher.h b/include/polarssl/cipher.h
index 9605066fd..02003e8cd 100644
--- a/include/polarssl/cipher.h
+++ b/include/polarssl/cipher.h
@@ -53,6 +53,7 @@ typedef enum {
POLARSSL_CIPHER_ID_DES,
POLARSSL_CIPHER_ID_3DES,
POLARSSL_CIPHER_ID_CAMELLIA,
+ POLARSSL_CIPHER_ID_BLOWFISH,
} cipher_id_t;
typedef enum {
@@ -78,14 +79,17 @@ typedef enum {
POLARSSL_CIPHER_CAMELLIA_256_CTR,
POLARSSL_CIPHER_DES_CBC,
POLARSSL_CIPHER_DES_EDE_CBC,
- POLARSSL_CIPHER_DES_EDE3_CBC
+ POLARSSL_CIPHER_DES_EDE3_CBC,
+ POLARSSL_CIPHER_BLOWFISH_CBC,
+ POLARSSL_CIPHER_BLOWFISH_CFB64,
+ POLARSSL_CIPHER_BLOWFISH_CTR,
} cipher_type_t;
typedef enum {
POLARSSL_MODE_NONE = 0,
POLARSSL_MODE_NULL,
POLARSSL_MODE_CBC,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
POLARSSL_MODE_OFB,
POLARSSL_MODE_CTR,
} cipher_mode_t;
@@ -121,8 +125,8 @@ typedef struct {
int (*cbc_func)( void *ctx, operation_t mode, size_t length, unsigned char *iv,
const unsigned char *input, unsigned char *output );
- /** Encrypt using CFB128 */
- int (*cfb128_func)( void *ctx, operation_t mode, size_t length, size_t *iv_off,
+ /** Encrypt using CFB (Full length) */
+ int (*cfb_func)( void *ctx, operation_t mode, size_t length, size_t *iv_off,
unsigned char *iv, const unsigned char *input, unsigned char *output );
/** Encrypt using CTR */
diff --git a/include/polarssl/cipher_wrap.h b/include/polarssl/cipher_wrap.h
index e81c11d09..4abbc4ef2 100644
--- a/include/polarssl/cipher_wrap.h
+++ b/include/polarssl/cipher_wrap.h
@@ -84,6 +84,18 @@ extern const cipher_info_t des_ede3_cbc_info;
#endif /* defined(POLARSSL_DES_C) */
+#if defined(POLARSSL_BLOWFISH_C)
+extern const cipher_info_t blowfish_cbc_info;
+
+#if defined(POLARSSL_CIPHER_MODE_CFB)
+extern const cipher_info_t blowfish_cfb64_info;
+#endif /* POLARSSL_CIPHER_MODE_CFB */
+
+#if defined(POLARSSL_CIPHER_MODE_CTR)
+extern const cipher_info_t blowfish_ctr_info;
+#endif /* POLARSSL_CIPHER_MODE_CTR */
+#endif /* defined(POLARSSL_BLOWFISH_C) */
+
#if defined(POLARSSL_CIPHER_NULL_CIPHER)
extern const cipher_info_t null_cipher_info;
#endif /* defined(POLARSSL_CIPHER_NULL_CIPHER) */
diff --git a/library/cipher_wrap.c b/library/cipher_wrap.c
index 99e509287..6b85903f0 100644
--- a/library/cipher_wrap.c
+++ b/library/cipher_wrap.c
@@ -45,6 +45,10 @@
#include "polarssl/des.h"
#endif
+#if defined(POLARSSL_BLOWFISH_C)
+#include "polarssl/blowfish.h"
+#endif
+
#include
#if defined(POLARSSL_AES_C)
@@ -157,7 +161,7 @@ const cipher_info_t aes_256_cbc_info = {
#if defined(POLARSSL_CIPHER_MODE_CFB)
const cipher_info_t aes_128_cfb128_info = {
POLARSSL_CIPHER_AES_128_CFB128,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
128,
"AES-128-CFB128",
16,
@@ -167,7 +171,7 @@ const cipher_info_t aes_128_cfb128_info = {
const cipher_info_t aes_192_cfb128_info = {
POLARSSL_CIPHER_AES_192_CFB128,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
192,
"AES-192-CFB128",
16,
@@ -177,7 +181,7 @@ const cipher_info_t aes_192_cfb128_info = {
const cipher_info_t aes_256_cfb128_info = {
POLARSSL_CIPHER_AES_256_CFB128,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
256,
"AES-256-CFB128",
16,
@@ -330,7 +334,7 @@ const cipher_info_t camellia_256_cbc_info = {
#if defined(POLARSSL_CIPHER_MODE_CFB)
const cipher_info_t camellia_128_cfb128_info = {
POLARSSL_CIPHER_CAMELLIA_128_CFB128,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
128,
"CAMELLIA-128-CFB128",
16,
@@ -340,7 +344,7 @@ const cipher_info_t camellia_128_cfb128_info = {
const cipher_info_t camellia_192_cfb128_info = {
POLARSSL_CIPHER_CAMELLIA_192_CFB128,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
192,
"CAMELLIA-192-CFB128",
16,
@@ -350,7 +354,7 @@ const cipher_info_t camellia_192_cfb128_info = {
const cipher_info_t camellia_256_cfb128_info = {
POLARSSL_CIPHER_CAMELLIA_256_CFB128,
- POLARSSL_MODE_CFB128,
+ POLARSSL_MODE_CFB,
256,
"CAMELLIA-256-CFB128",
16,
@@ -558,6 +562,118 @@ const cipher_info_t des_ede3_cbc_info = {
};
#endif
+#if defined(POLARSSL_BLOWFISH_C)
+
+int blowfish_crypt_cbc_wrap( void *ctx, operation_t operation, size_t length,
+ unsigned char *iv, const unsigned char *input, unsigned char *output )
+{
+ return blowfish_crypt_cbc( (blowfish_context *) ctx, operation, length, iv, input, output );
+}
+
+int blowfish_crypt_cfb64_wrap( void *ctx, operation_t operation, size_t length,
+ size_t *iv_off, unsigned char *iv, const unsigned char *input, unsigned char *output )
+{
+#if defined(POLARSSL_CIPHER_MODE_CFB)
+ return blowfish_crypt_cfb64( (blowfish_context *) ctx, operation, length, iv_off, iv, input, output );
+#else
+ ((void) ctx);
+ ((void) operation);
+ ((void) length);
+ ((void) iv_off);
+ ((void) iv);
+ ((void) input);
+ ((void) output);
+
+ return POLARSSL_ERR_CIPHER_FEATURE_UNAVAILABLE;
+#endif
+}
+
+int blowfish_crypt_ctr_wrap( void *ctx, size_t length,
+ size_t *nc_off, unsigned char *nonce_counter, unsigned char *stream_block,
+ const unsigned char *input, unsigned char *output )
+{
+#if defined(POLARSSL_CIPHER_MODE_CTR)
+ return blowfish_crypt_ctr( (blowfish_context *) ctx, length, nc_off, nonce_counter,
+ stream_block, input, output );
+#else
+ ((void) ctx);
+ ((void) length);
+ ((void) nc_off);
+ ((void) nonce_counter);
+ ((void) stream_block);
+ ((void) input);
+ ((void) output);
+
+ return POLARSSL_ERR_CIPHER_FEATURE_UNAVAILABLE;
+#endif
+}
+
+int blowfish_setkey_dec_wrap( void *ctx, const unsigned char *key, unsigned int key_length )
+{
+ return blowfish_setkey( (blowfish_context *) ctx, key, key_length );
+}
+
+int blowfish_setkey_enc_wrap( void *ctx, const unsigned char *key, unsigned int key_length )
+{
+ return blowfish_setkey( (blowfish_context *) ctx, key, key_length );
+}
+
+static void * blowfish_ctx_alloc( void )
+{
+ return malloc( sizeof( blowfish_context ) );
+}
+
+static void blowfish_ctx_free( void *ctx )
+{
+ free( ctx );
+}
+
+const cipher_base_t blowfish_info = {
+ POLARSSL_CIPHER_ID_BLOWFISH,
+ blowfish_crypt_cbc_wrap,
+ blowfish_crypt_cfb64_wrap,
+ blowfish_crypt_ctr_wrap,
+ blowfish_setkey_enc_wrap,
+ blowfish_setkey_dec_wrap,
+ blowfish_ctx_alloc,
+ blowfish_ctx_free
+};
+
+const cipher_info_t blowfish_cbc_info = {
+ POLARSSL_CIPHER_BLOWFISH_CBC,
+ POLARSSL_MODE_CBC,
+ 32,
+ "BLOWFISH-CBC",
+ 8,
+ 8,
+ &blowfish_info
+};
+
+#if defined(POLARSSL_CIPHER_MODE_CFB)
+const cipher_info_t blowfish_cfb64_info = {
+ POLARSSL_CIPHER_BLOWFISH_CFB64,
+ POLARSSL_MODE_CFB,
+ 32,
+ "BLOWFISH-CFB64",
+ 8,
+ 8,
+ &blowfish_info
+};
+#endif /* POLARSSL_CIPHER_MODE_CFB */
+
+#if defined(POLARSSL_CIPHER_MODE_CTR)
+const cipher_info_t blowfish_ctr_info = {
+ POLARSSL_CIPHER_BLOWFISH_CTR,
+ POLARSSL_MODE_CTR,
+ 32,
+ "BLOWFISH-CTR",
+ 8,
+ 8,
+ &blowfish_info
+};
+#endif /* POLARSSL_CIPHER_MODE_CTR */
+#endif /* POLARSSL_BLOWFISH_C */
+
#if defined(POLARSSL_CIPHER_NULL_CIPHER)
static void * null_ctx_alloc( void )
{
diff --git a/tests/CMakeLists.txt b/tests/CMakeLists.txt
index 5f768594f..a1e311fc5 100644
--- a/tests/CMakeLists.txt
+++ b/tests/CMakeLists.txt
@@ -37,6 +37,7 @@ add_test_suite(base64)
add_test_suite(blowfish)
add_test_suite(camellia)
add_test_suite(cipher cipher.aes)
+add_test_suite(cipher cipher.blowfish)
add_test_suite(cipher cipher.camellia)
add_test_suite(cipher cipher.des)
add_test_suite(cipher cipher.null)
diff --git a/tests/Makefile b/tests/Makefile
index 19f755fb9..13342f8cc 100644
--- a/tests/Makefile
+++ b/tests/Makefile
@@ -24,8 +24,9 @@ endif
APPS = test_suite_aes test_suite_arc4 \
test_suite_base64 test_suite_blowfish \
- test_suite_camellia \
- test_suite_cipher.aes test_suite_cipher.camellia \
+ test_suite_camellia test_suite_cipher.aes \
+ test_suite_cipher.blowfish \
+ test_suite_cipher.camellia \
test_suite_cipher.des test_suite_cipher.null \
test_suite_ctr_drbg test_suite_debug \
test_suite_des test_suite_dhm \
@@ -45,6 +46,10 @@ test_suite_cipher.aes.c : suites/test_suite_cipher.function suites/test_suite_ci
echo " Generate $@"
scripts/generate_code.pl suites test_suite_cipher test_suite_cipher.aes
+test_suite_cipher.blowfish.c : suites/test_suite_cipher.function suites/test_suite_cipher.blowfish.data scripts/generate_code.pl suites/helpers.function
+ echo " Generate $@"
+ scripts/generate_code.pl suites test_suite_cipher test_suite_cipher.blowfish
+
test_suite_cipher.camellia.c : suites/test_suite_cipher.function suites/test_suite_cipher.camellia.data scripts/generate_code.pl suites/helpers.function
echo " Generate $@"
scripts/generate_code.pl suites test_suite_cipher test_suite_cipher.camellia
@@ -93,6 +98,10 @@ test_suite_cipher.aes: test_suite_cipher.aes.c ../library/libpolarssl.a
echo " CC $@.c"
$(CC) $(CFLAGS) $(OFLAGS) $@.c $(LDFLAGS) -o $@
+test_suite_cipher.blowfish: test_suite_cipher.blowfish.c ../library/libpolarssl.a
+ echo " CC $@.c"
+ $(CC) $(CFLAGS) $(OFLAGS) $@.c $(LDFLAGS) -o $@
+
test_suite_cipher.camellia: test_suite_cipher.camellia.c ../library/libpolarssl.a
echo " CC $@.c"
$(CC) $(CFLAGS) $(OFLAGS) $@.c $(LDFLAGS) -o $@
diff --git a/tests/suites/test_suite_cipher.blowfish.data b/tests/suites/test_suite_cipher.blowfish.data
new file mode 100644
index 000000000..e3209a0d2
--- /dev/null
+++ b/tests/suites/test_suite_cipher.blowfish.data
@@ -0,0 +1,318 @@
+Cipher Selftest
+depends_on:POLARSSL_SELF_TEST
+cipher_selftest:
+
+Decrypt empty buffer
+dec_empty_buf:
+
+BLOWFISH Encrypt and decrypt 0 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:0
+
+BLOWFISH Encrypt and decrypt 1 byte
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:1
+
+BLOWFISH Encrypt and decrypt 2 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:2
+
+BLOWFISH Encrypt and decrypt 7 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:7
+
+BLOWFISH Encrypt and decrypt 8 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:8
+
+BLOWFISH Encrypt and decrypt 9 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:9
+
+BLOWFISH Encrypt and decrypt 15 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:15
+
+BLOWFISH Encrypt and decrypt 16 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:16
+
+BLOWFISH Encrypt and decrypt 17 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:17
+
+BLOWFISH Encrypt and decrypt 31 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:31
+
+BLOWFISH Encrypt and decrypt 32 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:32
+
+BLOWFISH Encrypt and decrypt 32 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:33
+
+BLOWFISH Encrypt and decrypt 47 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:47
+
+BLOWFISH Encrypt and decrypt 48 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:48
+
+BLOWFISH Encrypt and decrypt 49 bytes
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CBC:BLOWFISH-CBC:128:49
+
+BLOWFISH Encrypt and decrypt 0 bytes in multiple parts
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:0:0:
+
+BLOWFISH Encrypt and decrypt 1 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:1:0:
+
+BLOWFISH Encrypt and decrypt 1 bytes in multiple parts 2
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:0:1:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:16:0:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 2
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:0:16:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 3
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:1:15:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 4
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:15:1:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:15:7:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:16:6:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:17:6:
+
+BLOWFISH Encrypt and decrypt 32 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CBC:128:16:16:
+
+BLOWFISH Encrypt and decrypt 0 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:0
+
+BLOWFISH Encrypt and decrypt 1 byte
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:1
+
+BLOWFISH Encrypt and decrypt 2 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:2
+
+BLOWFISH Encrypt and decrypt 7 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:7
+
+BLOWFISH Encrypt and decrypt 8 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:8
+
+BLOWFISH Encrypt and decrypt 9 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:9
+
+BLOWFISH Encrypt and decrypt 15 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:15
+
+BLOWFISH Encrypt and decrypt 16 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:16
+
+BLOWFISH Encrypt and decrypt 17 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:17
+
+BLOWFISH Encrypt and decrypt 31 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:31
+
+BLOWFISH Encrypt and decrypt 32 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:32
+
+BLOWFISH Encrypt and decrypt 32 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:33
+
+BLOWFISH Encrypt and decrypt 47 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:47
+
+BLOWFISH Encrypt and decrypt 48 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:48
+
+BLOWFISH Encrypt and decrypt 49 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CFB64:BLOWFISH-CFB64:128:49
+
+BLOWFISH Encrypt and decrypt 0 bytes in multiple parts
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:0:0:
+
+BLOWFISH Encrypt and decrypt 1 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:1:0:
+
+BLOWFISH Encrypt and decrypt 1 bytes in multiple parts 2
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:0:1:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:16:0:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 2
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:0:16:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 3
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:1:15:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 4
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:15:1:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:15:7:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:16:6:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:17:6:
+
+BLOWFISH Encrypt and decrypt 32 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CFB
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CFB64:128:16:16:
+
+BLOWFISH Encrypt and decrypt 0 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:0
+
+BLOWFISH Encrypt and decrypt 1 byte
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:1
+
+BLOWFISH Encrypt and decrypt 2 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:2
+
+BLOWFISH Encrypt and decrypt 7 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:7
+
+BLOWFISH Encrypt and decrypt 8 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:8
+
+BLOWFISH Encrypt and decrypt 9 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:9
+
+BLOWFISH Encrypt and decrypt 15 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:15
+
+BLOWFISH Encrypt and decrypt 16 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:16
+
+BLOWFISH Encrypt and decrypt 17 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:17
+
+BLOWFISH Encrypt and decrypt 31 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:31
+
+BLOWFISH Encrypt and decrypt 32 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:32
+
+BLOWFISH Encrypt and decrypt 32 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:33
+
+BLOWFISH Encrypt and decrypt 47 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:47
+
+BLOWFISH Encrypt and decrypt 48 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:48
+
+BLOWFISH Encrypt and decrypt 49 bytes
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf:POLARSSL_CIPHER_BLOWFISH_CTR:BLOWFISH-CTR:128:49
+
+BLOWFISH Encrypt and decrypt 0 bytes in multiple parts
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:0:0:
+
+BLOWFISH Encrypt and decrypt 1 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:1:0:
+
+BLOWFISH Encrypt and decrypt 1 bytes in multiple parts 2
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:0:1:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:16:0:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 2
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:0:16:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 3
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:1:15:
+
+BLOWFISH Encrypt and decrypt 16 bytes in multiple parts 4
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:15:1:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:15:7:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:16:6:
+
+BLOWFISH Encrypt and decrypt 22 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:17:6:
+
+BLOWFISH Encrypt and decrypt 32 bytes in multiple parts 1
+depends_on:POLARSSL_BLOWFISH_C:POLARSSL_CIPHER_MODE_CTR
+enc_dec_buf_multipart:POLARSSL_CIPHER_BLOWFISH_CTR:128:16:16:
diff --git a/library/cipher.c b/library/cipher.c
index 0d1258bc9..f20cc73b4 100644
--- a/library/cipher.c
+++ b/library/cipher.c
@@ -86,6 +86,19 @@ static const int supported_ciphers[] = {
POLARSSL_CIPHER_DES_EDE3_CBC,
#endif /* defined(POLARSSL_DES_C) */
+#if defined(POLARSSL_BLOWFISH_C)
+ POLARSSL_CIPHER_BLOWFISH_CBC,
+
+#if defined(POLARSSL_CIPHER_MODE_CFB)
+ POLARSSL_CIPHER_BLOWFISH_CFB64,
+#endif /* defined(POLARSSL_CIPHER_MODE_CFB) */
+
+#if defined(POLARSSL_CIPHER_MODE_CTR)
+ POLARSSL_CIPHER_BLOWFISH_CTR,
+#endif /* defined(POLARSSL_CIPHER_MODE_CTR) */
+
+#endif /* defined(POLARSSL_BLOWFISH_C) */
+
#if defined(POLARSSL_CIPHER_NULL_CIPHER)
POLARSSL_CIPHER_NULL,
#endif /* defined(POLARSSL_CIPHER_NULL_CIPHER) */
@@ -168,6 +181,22 @@ const cipher_info_t *cipher_info_from_type( const cipher_type_t cipher_type )
return &des_ede3_cbc_info;
#endif
+#if defined(POLARSSL_BLOWFISH_C)
+ case POLARSSL_CIPHER_BLOWFISH_CBC:
+ return &blowfish_cbc_info;
+
+#if defined(POLARSSL_CIPHER_MODE_CFB)
+ case POLARSSL_CIPHER_BLOWFISH_CFB64:
+ return &blowfish_cfb64_info;
+#endif /* defined(POLARSSL_CIPHER_MODE_CFB) */
+
+#if defined(POLARSSL_CIPHER_MODE_CTR)
+ case POLARSSL_CIPHER_BLOWFISH_CTR:
+ return &blowfish_ctr_info;
+#endif /* defined(POLARSSL_CIPHER_MODE_CTR) */
+
+#endif
+
#if defined(POLARSSL_CIPHER_NULL_CIPHER)
case POLARSSL_CIPHER_NULL:
return &null_cipher_info;
@@ -247,6 +276,21 @@ const cipher_info_t *cipher_info_from_string( const char *cipher_name )
return cipher_info_from_type( POLARSSL_CIPHER_DES_EDE3_CBC );
#endif
+#if defined(POLARSSL_BLOWFISH_C)
+ if( !strcasecmp( "BLOWFISH-CBC", cipher_name ) )
+ return cipher_info_from_type( POLARSSL_CIPHER_BLOWFISH_CBC );
+
+#if defined(POLARSSL_CIPHER_MODE_CFB)
+ if( !strcasecmp( "BLOWFISH-CFB64", cipher_name ) )
+ return cipher_info_from_type( POLARSSL_CIPHER_BLOWFISH_CFB64 );
+#endif /* defined(POLARSSL_CIPHER_MODE_CFB) */
+
+#if defined(POLARSSL_CIPHER_MODE_CTR)
+ if( !strcasecmp( "BLOWFISH-CTR", cipher_name ) )
+ return cipher_info_from_type( POLARSSL_CIPHER_BLOWFISH_CTR );
+#endif /* defined(POLARSSL_CIPHER_MODE_CTR) */
+#endif
+
#if defined(POLARSSL_CIPHER_NULL_CIPHER)
if( !strcasecmp( "NULL", cipher_name ) )
return cipher_info_from_type( POLARSSL_CIPHER_NULL );
@@ -295,10 +339,10 @@ int cipher_setkey( cipher_context_t *ctx, const unsigned char *key,
#endif /* defined(POLARSSL_CIPHER_NULL_CIPHER) */
/*
* For CFB128 and CTR mode always use the encryption key schedule
*/
if( POLARSSL_ENCRYPT == operation ||
- POLARSSL_MODE_CFB128 == ctx->cipher_info->mode ||
+ POLARSSL_MODE_CFB == ctx->cipher_info->mode ||
POLARSSL_MODE_CTR == ctx->cipher_info->mode )
{
return ctx->cipher_info->base->setkey_enc_func( ctx->cipher_ctx, key,
@@ -421,9 +465,9 @@ int cipher_update( cipher_context_t *ctx, const unsigned char *input, size_t ile
return 0;
}
- if( ctx->cipher_info->mode == POLARSSL_MODE_CFB128 )
+ if( ctx->cipher_info->mode == POLARSSL_MODE_CFB )
{
- if( 0 != ( ret = ctx->cipher_info->base->cfb128_func( ctx->cipher_ctx,
+ if( 0 != ( ret = ctx->cipher_info->base->cfb_func( ctx->cipher_ctx,
ctx->operation, ilen, &ctx->unprocessed_len, ctx->iv,
input, output ) ) )
{
@@ -493,7 +537,7 @@ int cipher_finish( cipher_context_t *ctx, unsigned char *output, size_t *olen)
*olen = 0;
- if( POLARSSL_MODE_CFB128 == ctx->cipher_info->mode ||
+ if( POLARSSL_MODE_CFB == ctx->cipher_info->mode ||
POLARSSL_MODE_CTR == ctx->cipher_info->mode ||
POLARSSL_MODE_NULL == ctx->cipher_info->mode )
{
@@ -295,10 +339,10 @@ int cipher_setkey( cipher_context_t *ctx, const unsigned char *key,
#endif /* defined(POLARSSL_CIPHER_NULL_CIPHER) */
/*
- * For CFB128 and CTR mode always use the encryption key schedule
+ * For CFB and CTR mode always use the encryption key schedule
*/
if( POLARSSL_ENCRYPT == operation ||